/* -*- C -*-
 * Spamtest connection library
 * File: spamtest.h
 *      (C)  Alex Tutubalin <lexa@lexa.ru>
 *	and
 *	Mike Patutin <patutin@vist-v.ru>
 *	Apr-May 2003
 *
 * Created: Wed Apr 23 22:08:08 2003
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#include "msgstore.h"

#ifndef SPAM_FILTER_LIBRARY
#define SPAM_FILTER_LIBRARY

#ifdef __cplusplus
extern "C"
{
#endif

/* filter actions modes - change message/bounce message */
typedef enum 
{
    STS_ACTION_CHANGE=	1,
    STS_ACTION_BOUNCE=	2
} spamtest_reply_t;

/* change action types */
typedef enum
{
    STS_HEADER_NONE=	0,
    STS_HEADER_DEL=	1,
    STS_HEADER_CHG=	2,
    STS_HEADER_ADD=	3,
    STS_HEADER_NEW=	4,
    STS_HEADER_PREPEND=	5
} spamtest_action_t;

#define STS_ABUF_S 4096
/* timeouts in milliseconds */
#define STS_DEFAULT_CONNECT_TIMEOUT	1000
#define STS_DEFAULT_SND_RCV_TIMEOUT	50000

/* error codes */
/* if you add new code - don't forget to change sts_strerror function */
#define STS_ERR_NO_ERR			0
#define STS_ERR_OUT_OF_MEMORY		-1
#define STS_ERR_ILLEGAL_ADDRESS		-2
#define STS_ERR_UNABLE_TO_CONNECT	-3
#define STS_ERR_CONNECT_TIMEOUT		-4
#define STS_ERR_SND_RCV_TIMEOUT		-5
#define STS_ERR_INCORRECT_PROTOCOL	-6
#define STS_ERR_LIBRARY_ERR		-7
#define STS_ERR_UNABLE_TO_RESOLVE	-8
#define STS_ERR_SELECT_FD_ERR		-9
#define STS_ERR_READ_ERR	        -10
#define STS_ERR_ILLEGAL_CONFIG		-11
#define STS_ERR_BUFFER_NOT_PRESENT	-12
#define STS_ERR_BUFFER_IS_FULL		-14
#define STS_ERR_WRITE_ERR		-15
#define STS_ERR_GET_RECORD_SIZE		-16
#define STS_ERR_RECORD_TOO_LARGE	-17
#define STS_ERR_INVALID_PARAMS		-18
#define STS_ERR_NOT_INITED		-19

/* sts_prepare_envelope status codes */
#define STS_PREPARE_OK	  	  0
#define STS_ERR_ILLEGAL_RELAY	  1
#define STS_ERR_ILLEGAL_RCPT	  2
#define STS_ERR_ILLEGAL_MAIL_FROM 4


/* flags for sts_send_body/sts_send_envelope */ 
#define STS_REINIT_TIMEOUT	16
#define STS_FLUSH_DATA		32
#define STS_FINAL_CHUNK		64

/* flags for spamtest_session_t->flags */
#define STS_USE_POLL		1


/* session status types */
typedef enum  
{
    STS_SENDERROR = -2,	/* I/O or other error */
    STS_CONTINUE = -1,	/* should continue sending message */
    STS_ACCEPT=0,		/* message accepted, result should be analyzed
			   and actions on message performed */
    STS_FILTER_ERROR=1,	/* filter returns an error, message should be
			   rejected with 45x code */
    STS_REJECT=8,		/* filter rejects message, 55x code should be
			   returned to sender */
    STS_BLACKHOLE=16,	/* message blackholed - 220 code to sender, no
			   actions on message itself */
    STS_SMTP_ACCEPT=128	/* filter accepts message and have sent it by
			   SMTP - 220 OK to sender, no actions on message */
}
session_status_t;

typedef enum  
{
    STS_SMTP_MODE=	254,
    STS_ACTIONS_MODE=	253
}
filter_mode_t;

/* logger procedure definition */
struct __spamtest_session_t;
struct __msg_heap_t;
typedef int (* logger_proc)(struct __spamtest_session_t *s,int loglevel,const char * fmt,...);


/* this data structure describes header actions to be performed */

typedef struct hdraction_t
{
    spamtest_action_t	type;		/* STS_HEADER_ADD/DEL/NEW/CHG */
    char		*header;	/* header name */
    char		*value;		/* header value (not useful
					   with STS_HEADER_DEL */
}
hdraction_t;


/* describes message status returned by filter */

typedef struct message_status_t
{
    spamtest_reply_t	action;		/* STS_ACTION_CHANGE/BOUNCE */
    char		*mailfrom;	/* Envelope mail from for
					   generated message */
    recipient_list_t	*rcpts;		/* RCPT TO list for message */
    int			action_count;	/* count for header actions */
    hdraction_t		*act_array;	/* array of header actions */
    message_text_t	*bouncemsg;	/* message body for
					   STS_ACTION_BOUNCE */
    /* for internal use */
    int			action_alloc;
} message_status_t;


/* this data structure describes spamtest session status */

typedef struct
{
    session_status_t	status;		/* CONTINUE/ACCEPT/... */

    char		*reason;	/* for REJECT/ERROR - message
					   to be sent to sender in
					   SMTP chat */

    int			errorcode;	/* filter error code (use
					   sts_strerror() for
					   explanation */

    int			count;		/* for ACCEPT status - count
					   of generated messages */

    message_status_t	*ms_array;	/* array of generated message
					   statuses */
    /* for internal use */
    int			alloc;
    struct __msg_heap_t *storeptr;
}spamtest_status_t;


struct __bufio_t;

/* spam session definition */
typedef struct __spamtest_session_t {

    /* == THESE PARAMETERS ARE SET BY USER === */

    char *access_address;	/* "unix:/path/to/socket" or
				   "tcp:host:port" */
    char *mail_domain;		/* default mail domain for address
				 * canonification */

    size_t bufsize;		/* size of read/write buffers, default
				 * is 64k */

    logger_proc logp;		/* logger procedure */

    long rw_timeout;		/* read/write timeout in milliseconds */				
    long connect_timeout;	/* connect timeout in milliseconds */

    unsigned int flags;		/* STS_USE_POLL and so on */


    /* == THESE PARAMETERS SHOULD BE USED BY USER === */
    
    spamtest_status_t status;	/* session state (see spamtest_status_t
				 * definition for filed description */


    int recomended_size;	/* recommended data size to send in
				   sts_send_body at once */

    int should_store_message;	/* should user store local copy of
				 * message or filter will send mail by
				 * itself */

    char *message_id;		/* Message-ID generated by filter
				 * module */

    /* internal filter data, do not touch */
    int		filter_fd;
//    char	*relay_IP;
    char	*mail_from;
    long	timeout_curr;
    int		timeout_nq;
    filter_mode_t filter_mode; 		/* SMTP or ACTIONS */	
    recipient_list_t *rcpts;
    recipient_list_t *attrs;
    char *record_buf;

    /* flags */
    int envelope_sent;			/* 1=envelope already sent to filter*/
    int header_sent;			/* 1=header already sent to filter*/
    int body_sent;			/* 1=message body sent to
					 * filter (no more
					 * transmissions) */
    int chunk_sent;
    int rb_ptr;
    int line_complete;
    struct timeval tvs;			/* time when current operation
					 * start (do not modify!) */
    /* buffers */
    struct __bufio_t *bufio;
    struct __msg_heap_t *store;
    unsigned char attr_buf[STS_ABUF_S];

    unsigned int magic;			/* sentinel */
} spamtest_session_t;


/* ==== LIBRARY CALLS ==== */

/* converts numeric STS_* error code into string */
const char *sts_strerror(int ercode);

/* initialinig and start connection */
int sts_init(spamtest_session_t *session);
int sts_reuse(spamtest_session_t *session);

// use poll instead of select()

/*
   sts_prepare_envelope and others:
   Initializes part (or full) of message envelope	   
   DOES NOT SEND anything to filter, prepare data only
   sts_prepare_envelope returns additional status in *status field	       
*/
int sts_set_sender(spamtest_session_t *s, const char *mail_from);
int sts_add_recipient(spamtest_session_t *s, const char *rcpt);
int sts_set_relay(spamtest_session_t *s, const char *relay_IP);
/* all of the above */
int sts_prepare_envelope(spamtest_session_t *s, 
			 const char* mail_from,  
			 const char * rcpt,
			 const char *relay_IP,
			 int *stat); 

int sts_add_attribute(spamtest_session_t *s, const char *attrname, const char *attrval);

/*
  sts_send_body:
  sends part of mail message to filtering engine.

  Returns:
  NULL if invalid or uninitialized session data specified
  spamtest_status_t* structure overwise
  
  msgbody points to part of message data (headers and body),
  msglen  sets data length
  flags   parameter is a bit mask with fields:
		STS_REINIT_TIMEOUT - reinit timeout, discard old
		execution statistics
		STS_FLUSH_DATA - force flushing of data to filter and
				reanalysing 
		STS_FINAL_CHUNK - this call of sts_send_body is final
		(calls sts_body_end internally)				

*/
spamtest_status_t* sts_send_body(spamtest_session_t *session,
				 const char *msgbody,
				 size_t msglen,
				 int flags);

/*
  sts_body_end:
  flushes I/O buffers and forces filter to return message status

  Returns:
  NULL if invalid or uninitialized session data specified
  spamtest_status_t* structure overwise
  flags   parameter is a bit mask with fields:
		STS_REINIT_TIMEOUT - reinit timeout, discard old
		execution statistics
		STS_FLUSH_DATA - force flushing of data to filter and
				reanalysing 

*/
spamtest_status_t* sts_body_end(spamtest_session_t *session,
				int flags);

/*
  sts_close:
  closes all sockets, deallocates all memory.
  Returns: 
  STS_ERR_ code (use sts_strerror for explanation)
*/

int sts_close(spamtest_session_t *session);
void sts_library_init(void);


/* === Utility and diagnostic functions === */

/* actions/status manip */

void dump_message_status(spamtest_status_t *smsg);
headers_list_t* make_changed_headers(message_status_t * sts, message_t *message);
spamtest_status_t* glue_actions(spamtest_status_t *s);


extern int sts_default_logger_log_level;

int rfc822_unquote_one(char *dest,unsigned int size, char *from, char *defdomain);
int rfc822_quote_one(char *dest,unsigned int size, char *from, char *defdomain);
recipient_list_t *rfc822_unquote_list(recipient_list_t *from,char *defdomain);
recipient_list_t *rfc822_quote_list(recipient_list_t *from,char *defdomain);


#ifdef __cplusplus
}
#endif

#endif /* SPAM_FILTER_LIBRARY */
